#!/usr/bin/python3

"""
scan a music directory to find concerts near a specified location
"""

from .version import __version__
from .cli import parse_args
from .scan import scan_wrapper
from .concerts import bit
from .output import output_wrapper


def main():
    """Main function."""
    args = parse_args()
    raw_artists = scan_wrapper(args.directory, args.cache)
    concert_list = bit(raw_artists, args)
    output_wrapper(concert_list, args.output)


if __name__ == "__main__":
    main()



#!/usr/bin/python3

"""
format concerts data into final output and write it to disk
"""

import os
import json

from datetime import datetime

import feedgenerator as fg


def pretty_strings(concert):
    """Extract relevant data from concert dict and format it"""
    event_id = concert['id']
    artists = ", ".join(map(str, concert['lineup']))
    date = datetime.fromisoformat(concert['datetime']).strftime("%Y-%m-%d %H:%M")
    venue = concert['venue']['name'] + ", " + concert['venue']['city']
    description = concert['description']
    url = 'https://www.bandsintown.com/e/' + event_id
    return event_id, artists, date, venue, description, url


def atom(concert_list):
    """Convert concerts to ATOM format"""
    output = fg.Atom1Feed(title = 'Metalfinder',
                          description = 'Feed for your personalised concerts',
                          link = 'https://gitlab.com/baldurmen/metalfinder',
                          language = 'en')
    # This is needed for new concerts to be at the top of the ATOM file
    concert_list = sorted(concert_list,
                          key=lambda concert: concert['updated'],
                          reverse=True)
    for concert in concert_list:
        event_id, artists, date, venue, description, url = pretty_strings(concert)
        output.add_item(title = f"{artists} @ {venue} on {date}",
                        link = url,
                        content = description,
                        description = description,
                        unique_id = event_id,
                        pubdate = concert['updated'])
    return output


def txt(concert_list):
    """Convert concerts to text format"""
    output = ''
    for concert in concert_list:
        dummy, artists, date, venue, description, url = pretty_strings(concert)
        output += ('\n'
                   f'Artists: {artists},\n'
                   f'Date: {date},\n'
                   f'Venue: {venue},\n'
                   f'Description: {description}\n'
                   f'Link: {url},\n'
                   '\n'
                   '===============================================================\n')
    return output


def _json(concert_list):
    """Convert concerts to JSON format"""
    output = []
    for concert in concert_list:
        event_id, artists, date, venue, description, url = pretty_strings(concert)
        output.append({"eventId": event_id, "artists": artists, "date": date,
                       "venue": venue, "description": description, "link": url})
    output = json.dumps(output)
    return output


def output_wrapper(concert_list, output_path):
    """Wrapper function to manage the output"""
    extension = os.path.splitext(output_path)[1][1:]
    if extension == 'atom':
        output = atom(concert_list)
        with open(output_path, 'w', encoding='utf-8') as final:
            output.write(final, 'utf-8')
    else:
        if extension == 'txt':
            output = txt(concert_list)
        if extension == 'json':
            output = _json(concert_list)
        with open(output_path, 'w', encoding='utf-8') as final:
            final.write(output)



#!/usr/bin/python3

"""
CLI arguments comprehension
"""

import os
import sys
import argparse

from datetime import date
from pathlib import Path
from xdg.BaseDirectory import xdg_cache_home


__version__ = "1.0.0"


def output_choices(fname):
    """Restrict the output extension choices"""
    extension = os.path.splitext(fname)[1][1:]
    choices = ['txt', 'json', 'atom']
    if extension not in choices:
        sys.exit(f"output '{fname}' doesn't end with a supported extension."
                 f"Choose from {*choices, }")
    if not os.path.isdir(Path(fname).parent):
        sys.exit(f"output '{fname}' is not a valid path.")
    return fname


def date_lint(mdate):
    """Verify date is ISO 8601 and exit nicely if not"""
    try:
        date.fromisoformat(mdate)
    except ValueError:
        sys.exit(f"max-date '{mdate}' does not follow ISO 8601 (YYYY-MM-DD) format.")
    if date.today() > date.fromisoformat(mdate):
        sys.exit(f"max-date '{mdate}' should not be ealier than today ({date.today()})")
    return mdate


def dir_lint(path):
    """Verify path is a valid directory and exit nicely if not"""
    if not os.path.isdir(path):
        if path == os.path.join(xdg_cache_home, 'metalfinder'):
            os.mkdir(path)
        else:
            sys.exit(f"Path '{path}' is not valid.")
    return path


def parse_args():  # pylint: disable=W0613
    """Parse the arguments"""
    example_text = '''example:
        metalfinder -d "/home/foo/Music" -o "/home/foo/metalfinder.atom" -l "Montreal" -b "foo"'''
    mf_cache_dir = os.path.join(xdg_cache_home, 'metalfinder')
    parser = argparse.ArgumentParser(prog='metalfinder',
                                     formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description=__doc__,
                                     epilog=example_text)
    parser.add_argument('-d', '--directory', type=dir_lint, required=True,
                        help='music directory to scan to create artist list')
    parser.add_argument('-o', '--output', type=output_choices,
                        required=True, help='path to the desired output file')
    parser.add_argument('-l', '--location', type=str, required=True,
                        help='name of the city to use when looking for concerts')
    parser.add_argument('-b', '--bit-appid', type=str, required=True,
                        help='Bandsintown App ID (API key)')
    parser.add_argument('-c', '--cache', type=dir_lint,
                        default=mf_cache_dir, help='path to the cache directory')
    parser.add_argument('-m', '--max-date', type=date_lint,
                        help='max date in YYYY-MM-DD format (ISO 8601)')
    parser.add_argument('--version', action='version', version='%(prog)s ' + __version__)
    parser.add_argument('--verbose', action='store_true',
                        help='run the program in verbose mode')
    return parser.parse_args()



#!/usr/bin/python3

"""
scan a music directory and output a list of artists
"""

import os
import re
import pickle

import mutagen


def has_changed(song_cache, new_song_cache, fullpath):
    """Query song cache to know if file has changed (or is new) since last run.
    In any case, update the cache"""
    file_changed = False
    mtime = os.stat(fullpath).st_mtime
    if (fullpath not in song_cache) or (fullpath in song_cache and
                                        song_cache[fullpath] != mtime):
        file_changed = True
    new_song_cache[fullpath] = mtime
    return new_song_cache, file_changed


def write_song_cache(data, cache_dir):
    """Write song cache file to disk"""
    song_cache_file = os.path.join(cache_dir, 'song_cache')
    with open(song_cache_file, 'wb') as _cache:
        pickle.dump(data, _cache)


def get_song_cache(cache_dir):
    """Get the song cache if it exists"""
    song_cache = {}
    song_cache_file = os.path.join(cache_dir, 'song_cache')
    if os.path.isfile(song_cache_file):
        with open(song_cache_file, 'rb') as _cache:
            song_cache = pickle.load(_cache)
    return song_cache


def write_artist_cache(data, cache_dir):
    """Write artist cache file to disk"""
    artist_cache_file = os.path.join(cache_dir, 'artist_cache')
    with open(artist_cache_file, 'w+', encoding='utf-8') as _cache:
        _cache.write('\n'.join(data))


def get_artist_cache(cache_dir):
    """Get the artist cache if it exists"""
    raw_artists = []
    artist_cache_file = os.path.join(cache_dir, 'artist_cache')
    if os.path.isfile(artist_cache_file):
        with open(artist_cache_file, 'r', encoding='utf-8') as _cache:
            raw_artists = _cache.read().splitlines()
    return raw_artists


def modify_artist(fullpath, raw_artists, action):
    """Modify artist list"""
    try:
        artist = mutagen.File(fullpath, easy=True)["artist"]
        artist = ''.join(artist)  # list to string
        split_pattern = re.compile(r"\+|/|&")
        if re.search(split_pattern, artist):
            splitted_artist = re.split(split_pattern, artist)
            for artist in splitted_artist:
                if action == 'add':
                    raw_artists.append(artist.lstrip())
                elif action == 'remove':
                    raw_artists.remove(artist.lstrip())
        else:
            if action == 'add':
                raw_artists.append(artist)
            elif action == 'remove':
                raw_artists.remove(artist)
    except KeyError:  # no artist tag
        pass
    except mutagen.flac.FLACNoHeaderError:  # FLAC file has no header
        pass
    return raw_artists


def scan_dir(music_dir, song_cache, raw_artists):
    """Scan a directory and output a list of artists"""
    new_song_cache = {}
    for dirname, _, filenames in os.walk(music_dir, topdown=False):
        for song in filenames:
            fullpath = os.path.abspath(os.path.join(dirname, song))
            if song.endswith(('.flac', '.mp3', '.ogg')):
                new_song_cache, file_changed = has_changed(song_cache, new_song_cache, fullpath)
                if file_changed:
                    raw_artists = modify_artist(fullpath, raw_artists, 'add')
    stale_song_cache = set(song_cache) - set(new_song_cache)
    for song in stale_song_cache:
        raw_artists = modify_artist(song, raw_artists, 'remove')
    return set(raw_artists), new_song_cache


def scan_wrapper(music_dir, cache_dir):
    """Wrapper function to manage the scan"""
    raw_artists = get_artist_cache(cache_dir)
    song_cache = get_song_cache(cache_dir)
    raw_artists, new_song_cache = scan_dir(music_dir, song_cache, raw_artists)
    write_song_cache(new_song_cache, cache_dir)
    write_artist_cache(raw_artists, cache_dir)
    return raw_artists



#!/usr/bin/python3

"""
get concerts data from API provider
"""

import os
import pickle
import datetime

from .api.bandsintown import Client


def has_changed(concert_cache, concert):
    """Query concert cache to know if concert has changed (or is new) since
    last run."""
    concert_is_new = False
    if not any(concert['id'] in item['id'] for item in concert_cache):
        concert_is_new = True
    return concert_is_new


def write_concert_cache(data, cache_dir):
    """Write concert cache file to disk"""
    concert_cache_file = os.path.join(cache_dir, 'concert_cache')
    with open(concert_cache_file, 'wb') as _cache:
        pickle.dump(data, _cache)


def get_concert_cache(cache_dir):
    """Get the concert cache if it exists"""
    concert_cache = []
    concert_cache_file = os.path.join(cache_dir, 'concert_cache')
    if os.path.isfile(concert_cache_file):
        with open(concert_cache_file, 'rb') as _cache:
            concert_cache = pickle.load(_cache)
    return concert_cache


def query_bit(raw_artists, args):
    """Query Bandsintown for concert list"""
    bit_client = Client(args.bit_appid)
    concert_list = []
    for artist in raw_artists:
        # TODO: Why are we getting some empty artists?
        if artist != '':
            if args.max_date:
                date_range = str(datetime.date.today()) + ',' + args.max_date
                concerts = bit_client.artists_events(artist, date=date_range)
            else:
                concerts = bit_client.artists_events(artist)
            if concerts is not None:
                concert_list = concert_list + concerts
    return concert_list


def filter_location(concert_list, location):
    """Filter concerts based on location"""
    concert_list = [concert for concert in concert_list if
            (concert['venue']['city'] == location)]
    return concert_list


def bit(raw_artists, args):
    """Wrapper function for Bandsintown provider"""
    concert_cache = get_concert_cache(args.cache)
    concert_list = query_bit(raw_artists, args)
    concert_list = filter_location(concert_list, args.location)
    for concert in concert_list:
        if has_changed(concert_cache, concert):
            concert['updated'] = datetime.datetime.now()
            concert_cache.append(concert)
    write_concert_cache(concert_cache, args.cache)
    return concert_cache
